/**
 *  Every fork holds information about its owner -- the philosopher who currently uses it. A fork can be given
 *  to a philosopher and released via its synchronized methods only.
 *
 *  @author <a href="http://home.zcu.cz/~jkacer">Jarda Kacer</a>, Czech OS/2 User Group; email: <a href="mailto:jkacer@kiv.zcu.cz">jkacer@kiv.zcu.cz</a>
 *
 */
public class Fork
{
 private Philosopher    owner;


 /**
  *  Creates a new fork with no owner.
  *
  */
 public Fork()
 {
  owner = null;
 } // constructor


 /**
  *  Returns the fork's owner.
  *
  */
 public synchronized Philosopher getOwner()
 {
  return owner;
 } // getOwner


 /**
  *  Gives the fork to the philosopher requiring it. If the fork already belongs to another philosopher,
  *  the calling thread is blocked until the fork is released.
  *
  */
 public synchronized void acquireFork(Philosopher philosopher)
 {
  while (owner != null)
  {
   try
   {
    wait();
   } // try
   catch (InterruptedException e)
   {
    System.err.println("A philosopher interrupted while waiting for a fork!");
   } // catch
  } // while
  owner = philosopher;
 } // acquireFork


 /**
  *  Releases the fork and wakes up a blocked thread waiting for the fork, if there is any.
  *
  */
 public synchronized void releaseFork(Philosopher philosopher)
 {
  if (owner != philosopher)
   return;
  else
  {
   owner = null;
   notify();
  }
 } // releaseFork
} // class Fork

